#include "Utils.h"

#include <bits/stdc++.h>

/**
 * @brief Gets index of number in vector.
 * @param vec Vector to find number in
 * @param function number to find.
 * @return -1 if number is not found. -2 if number is repeated. Index of element if number is found.
 **/
int VecUtils::getIndex(std::vector<int>& vec, int function) {
  int c = std::count(vec.begin(), vec.end(), function);
  if (c == 0) return -1;
  if (c > 1) return -2;

  auto it = std::find(vec.begin(), vec.end(), function);

  if (it != vec.end()) return std::distance(vec.begin(), it);

  return -1;
}

/**
 * @brief Gets two indexes of number in vector<vector<int>>.
 * @param vec Vector to find number in
 * @param function number to find.
 * @param hyb_index reference to put the hybrid index in.
 * @param vmm_index reference to put the vmm index in.
 * @return -1 if number is not found. -2 if number is repeated. 0 if number is found.
 **/
int VecUtils::getIndex(std::vector<std::vector<int>>& vec, int function, int& hyb_index, int& vmm_index) {
  hyb_index = -1;
  vmm_index = -1;

  int occurrences = 0;

  for (size_t i = 0; i < vec.size(); ++i) {
    auto& subvec = vec[i];
    auto it = std::find(subvec.begin(), subvec.end(), function);

    if (it != subvec.end()) {
      ++occurrences;
      if (occurrences > 1) {
        hyb_index = -2;
        vmm_index = -2;
        return -2;  // Duplicated
      }

      hyb_index = static_cast<int>(i);
      vmm_index = static_cast<int>(std::distance(subvec.begin(), it));
    }
  }

  if (occurrences == 0) {
    hyb_index = -1;
    vmm_index = -1;
    return -1;  // Not found
  }

  return 0;  // Found exactly once
}

/**
 * @brief Converts a timestamp to a string.
 */
std::string TimeUtils::timeToStr(std::chrono::time_point<std::chrono::system_clock> timestamp) {
  std::time_t time_t_value = std::chrono::system_clock::to_time_t(timestamp);
  std::tm tm_value = *std::localtime(&time_t_value);
  std::ostringstream oss;
  oss << std::put_time(&tm_value, "%Y-%m-%d %H:%M:%S");
  return oss.str();
}